/*******************************************************************************
 * Copyright (c) 2015-2016, IncQuery Labs Ltd. and Ericsson AB
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Akos Horvath, Abel Hegedus, Zoltan Ujhelyi, Daniel Segesdi, Tamas Borbas, Robert Doczi, Peter Lunk, Istvan Papp - initial API and implementation
 *******************************************************************************/
package com.incquerylabs.emdw.cpp.codegeneration.templates

import com.ericsson.xtumlrt.oopl.cppmodel.CPPPackage
import org.eclipse.incquery.runtime.api.IncQueryEngine

class PackageTemplates extends CPPTemplate {
	extension val NamespaceTemplates namespaceTemplates
	extension val HeaderGuardTemplates headerGuardTemplates
	extension val IncludeTemplates includeTemplates
	extension val EnumTemplates enumTemplates
	extension val StructTemplates structTemplates
	
	new(IncQueryEngine engine) {
		super(engine)
		namespaceTemplates = new NamespaceTemplates
		headerGuardTemplates = new HeaderGuardTemplates
		includeTemplates = new IncludeTemplates
		enumTemplates = new EnumTemplates
		structTemplates = new StructTemplates
	}
	
	def packageMainHeaderTemplate(CPPPackage cppPackage) {
		val cppPackageName = cppPackage.cppName
		val headerGuardPostfix = "MAIN_HEADER"
		
		'''
		/******************************************************************************
		 *
		 * P A C K A G E   M A I N   ( H E A D E R )
		 *
		 * NAME: «cppPackageName»
		 *
		 *
«««		 TODO: generated by [...]
		 *
		 ******************************************************************************/
		«startHeaderGuard(cppPackage, headerGuardPostfix)»
		
		«cppPackage.headerFile.inclusions»
		
		«closeHeaderGuard(cppPackage, headerGuardPostfix)»
		'''
	}
	
	def packageMainBodyTemplate(CPPPackage cppPackage) {
		val cppPackageName = cppPackage.cppName
		
		'''
		/******************************************************************************
		 *
		 * P A C K A G E   M A I N   ( B O D Y )
		 *
		 * NAME: «cppPackageName»
		 *
		 *
«««		 TODO: generated by [...]
		 *
		 ******************************************************************************/
		
		«cppPackage.bodyFile.inclusions»
		
		'''
	}
}
